      INTEGER FUNCTION ENCODE(FIELD,VALUE,IPTR,IDECC,NPTR)
C
C     I C H z E N C O D E
C
C     Subroutine in which the real variable VALUE is placed
C     into the character string FIELD in a nice way;
C     that is, it may be purely integral (within the integer
C     16 bit range), or it may be so large or so small that
C     'E' format is best.  Generally, an ordinary floating
C     point format is used.
C
C     Parameters -  (">" input, "<" output)
C
C     (>) FIELD   (Character) The character string into which
C                 VALUE is to be encoded.
C
C     (>) VALUE   (Real) The number to be encoded
C
C     (>) IPTR    (Integer) quantity giving the number of
C                 the first byte in FIELD (starting at 1) into
C                 which the ASCII encoded value is to be written.
C
C     (>) IDDEC   (Integer) Number of decimal places to be used.
C
C     (<) NPTR    (Integer) Variable in which ENCODE
C                 returns the number of the next available byte
C                 in FIELD.
C
C     Returns - (if called as a function)
C
C     (<)ENCODE (Integer) Return code.
C                   0 => OK,
C                   1 => Not enough character positions left
C                        in FIELD.
C                       
C     Based on a program written by Tony Hewer at UCL
C
C                                   KS / CIT  14th Nov 1982
C+

      CHARACTER*(*) FIELD
      INTEGER IPTR,NPTR,IDECC
C not used INTEGER NL, NOK
      CHARACTER MINUS,IPOINT,IEE
      LOGICAL LEAD
      DATA MINUS,IPOINT,IEE/'-','.','E'/

C
      NLB=LEN(FIELD)
      IZERO=ICHAR('0')
	DO I=1,NLB
C		FIELD(I:I)='\0'
		FIELD(I:I)=' '
	END DO
C
C     If value is within integer range, print in ordinary
C     decimal form, otherwise print in '1PE' format.
C
C     Look if within integer range
C
      ENCODE=0
      IDEC=IDECC
      IPTR1=IPTR
      IEXS=0
      DECP=FLOAT(IDECC)
      DECM=10.**(-DECP)
      DECM1=10.**(DECP)
      DECM2=5.*10.**(-DECP-1.)
      VAL1=ABS(VALUE)
      VAL2=VALUE
      VALUE1=VAL1
      IF (VAL1.EQ.0.)   GO TO 50
      IF ((VAL1.LT.32767.).AND.(VAL1.GE.DECM))   GO TO 50
C
C     Work out exponential etc for 1PE format
C
      IP=1
      IF (VAL1.LT.1.)   IP=-1
      DO 40 I=1,79
         DIV=10.**FLOAT(I*IP)
         VAL=VAL1/DIV
         IF (IP.LT.0)   GO TO 30
            IF (VAL.LT.1.)   GO TO 42
            GO TO 40
   30    IF (VAL.GE.1.)   GO TO 42
   40    CONTINUE
   42 IEXS=I-1
      VALUE1=VAL*10.
      IF (IP.GE.1)  GO TO 50
         IEXS=-I
         VALUE1=VAL
C
C     Now get integer and decimal part
C
   50 INTPAR=VALUE1
      RINT=FLOAT(INTPAR)
      DECPAR=ABS(VALUE1-RINT)
      IF (DECPAR.EQ.0.)   GO TO 60
      DECPAR=DECPAR+DECM2
      IF (DECPAR.LT.1.)   GO TO 60
         INTPAR=INTPAR+1
         DECPAR=0.
C
C     If number negative, generate '-' sign
C
   60 IF (VAL2.GE.0.)   GO TO 80
         IF (IPTR1.GT.NLB)   GO TO 350
            FIELD(IPTR1:IPTR1)=MINUS
            IPTR1=IPTR1+1
C
C     If integer part zero, next one is a '0'
C
   80 IF (INTPAR.NE.0)   GO TO 100
         IF (IPTR1.GT.NLB)   GO TO 350
         FIELD(IPTR1:IPTR1)='0'
         IPTR1=IPTR1+1
C
C     And if no decimal part, go test for exponential
C
      IF (DECPAR.EQ.0.)   GO TO 250
         GO TO 180
C
C     Now generate number string for integer part
C
  100 IB=0
      LEAD=.FALSE.
      DO 150 I=1,5
         IDIV=10**(5-I)
         IVAL=INTPAR/IDIV
         IF (LEAD)  GO TO 125
            IF (IVAL.EQ.0)   GO TO 150
               LEAD=.TRUE.
  125          NUMB=IVAL-IB
               NUMB=MAX0(NUMB,0)
               NUMB=MIN0(NUMB,9)
               IF (IPTR1.GT.NLB)   GO TO 350
               FIELD(IPTR1:IPTR1)=CHAR(NUMB+IZERO)
               IPTR1=IPTR1+1
               IB=IVAL*10
  150    CONTINUE
C
C     Look if decimal part required
C
      IF (DECPAR.EQ.0.)   GO TO 250
C
C     Yes.  First generate a decimal point.
C
  180    IF (IPTR1.GT.NLB)   GO TO 350
         FIELD(IPTR1:IPTR1)=IPOINT
         IPTR1=IPTR1+1
C
C        Generate number string for decimal point
C
         DO 220 I=1,IDEC
            DECPAR=DECPAR*10.
            NUMB=DECPAR
            IVAL=NUMB
            NUMB=MAX0(NUMB,0)
            NUMB=MIN0(NUMB,9)
            IF (IPTR1.GT.NLB)   GO TO 350
            FIELD(IPTR1:IPTR1)=CHAR(NUMB+IZERO)
            IPTR1=IPTR1+1
            DECPAR=DECPAR-FLOAT(IVAL)
  220       CONTINUE
C
C     Exponential needed?
C
  250 IF (IEXS.EQ.0)   GO TO 300
C
C        Yes.  Generate 'E'
C
         IF (IPTR1.GT.NLB)   GO TO 350
         FIELD(IPTR1:IPTR1)=IEE
         IPTR1=IPTR1+1
C
C        If exponential is negative, also generate '-' sign
C
         IF (IEXS.GE.0)   GO TO 270
            IF (IPTR1.GT.NLB)   GO TO 350
            FIELD(IPTR1:IPTR1)=MINUS
            IPTR1=IPTR1+1
            IEXS=-IEXS
C
C         Generate exponential number string
C
  270    IB=IEXS/10
         IVAL=IEXS-IB*10
         IF (IPTR1.GT.NLB)   GO TO 350
         FIELD(IPTR1:IPTR1)=CHAR(IB+IZERO)
         IPTR1=IPTR1+1
         IF (IPTR1.GT.NLB)   GO TO 350
         FIELD(IPTR1:IPTR1)=CHAR(IVAL+IZERO)
         IPTR1=IPTR1+1
C
C     Set new char-pointer and return
C
  300 NPTR=IPTR1
      GO TO 600
C
C     Bad return - buffer filled
C
  350 ENCODE=1
C
  600 CONTINUE
      END
