c  Subroutine to smooth the outer region of a psf using azimuthal
c       averaging over sections.
c         adopted to ppp April 1991 from lissazim by D. Nadeau
c
****************************************************************
*                                                              *
*                                                              *
*   D. Nadeau                                                  *
*   Universite de Montreal                    9 JUILLET 1989   *
*                                 Revise     23 AOUT    1989   *
*                                                              *
*   Fait un lissage en azimut des contours les plus bas d'une  *
*   PSF. Tire avantage du fait que les variations de la PSF    *
*   en azimut ont une frequence spatiale plus faible que les   *
*   variations radiales. Ceci permet de faire un lissage de    *
*   la PSF sans modifier son profil radial. Dans la presente   *
*   version le lissage azimutal n'est pas parfaitement separe  *
*   du lissage radial, a cause des etendues radiales dif-      *
*   ferentes des pixels qui entrent dans le calcul de la       *
*   moyenne pour un segment de pixel donne.                    *
*   Il faudrait associer a chaque segment de pixel,            *
*   a l'interieur de l'un des anneaux minces en lesquels       *
*   est divisee la region de lissage, une valeur interpolee    *
*   selon les variations radiales a cet azimut.                *
*                                                              *
*   Le programme demande les coordonnees du centre de la PSF,  *
*   la largeur et l'etendue azimutale de la section d'anneau   *
*   sur lequel est calculee la valeur moyenne, et trois        *
*   valeurs de rayon.                                          *
*                                                              *
*   Le lissage est fait sur le nombre de degres specifies      *
*   entre le cercle de rayon intermediaire et le cercle de     *
*   rayon maximum.                                             *
*   Dans la region entre le cercle intermediaire et le cercle  *
*   minimum, le lissage est fait sur un nombre de degres       *
*   variant entre 2.*ASIND(0.708/raymin) au cercle minimum     *
*   et le nombre specifie au cercle intermediaire.             *
*   Le nombre minimum de degres de lissage est fixe ainsi      *
*   pour s'assurer que le lissage se fait sur au moins un      *
*   peu plus d'un pixel.                                       *
*                                                              *
*   Les coordonnees d'un pixel designent la position du        *
*   centre du pixel.                                           *
*                                                              *
*   Cette version ne prevoit pas la possibilite que les        *
*   cercles de lissage sortent par endroits de l'image.        *
*                                                              *
*   23 aout 1989: Modification pour conserver le flux.         *
*                                                              *
*   Pour utiliser ce programme sur un ordinateur autre que     *
*   VAX, utiliser la fonction appropriee pour trouver          *
*   l'entier a 2 octets le plus proche d'un reel a 8 octets,   *
*   soit l'equivalent de la fonction VAX IIDNNT.               *
*                                                              *
c
c  driver routine
c
	subroutine smopsfd(*,nfr,ix,nu,nv,kf)
c	integer*2 ix(nu,nv,kf)
	integer ix(nu,nv,kf)
	call querynfr(nfr,kf,0,*999,*101)
 101	call smopsf(nfr,ix(1,1,nfr),nu,nv)
 999	return1
	end
c
	subroutine smopsf(nfr,image,nu,nv)
      parameter (NAXEI=256)
      parameter (NAXEJ=256)
*                                                              *
****************************************************************

c     common /unites/nur1,nuw1
c     common /INTRAC/nur,nuw
c     integer*2 image(nu,nv)
      integer image(nu,nv)
      integer*2 ierr
      integer*2 itmp, jtmp

      real*4 tmp,tmp4
      real*4 ci, cj, xi, xj

*  rayons et angles de lissage correspondants:
      real*4 rayon(20), azim(20)   
*  image de depart:
      real*8 brute(250,250)
*  somme des fractions de pixel lissees:
c      real*8 poidsval(NAXEI,NAXEJ)
       real*8 poidsval(250,250)
*  somme des "valeur de lissage"*"frac. de pixel" 
      real*8 vallis(250,250)
*  Pour pixels interceptes par "ray1", "ray2" et "union":
      integer*2 nray1, nray2, nuni
      integer*2 kray1, kray2, kuni
*  Coor. "ii" et "jj" des pixels:
c      integer*2 ii1(2*(NAXEI+NAXEJ)), jj1(2*(NAXEI+NAXEJ))
c      integer*2 ii2(2*(NAXEI+NAXEJ)), jj2(2*(NAXEI+NAXEJ))
c      integer*2 iiuni(4*(NAXEI+NAXEJ)), jjuni(4*(NAXEI+NAXEJ))
      integer*2 ii1(1000), jj1(1000)
      integer*2 ii2(1000), jj2(1000)
      integer*2 iiuni(2000), jjuni(2000)
*  Frac. du pix. interieure au cercle:
      real*4    frac1(1000), frac2(1000)
*  Frac. du pix. entre les deux cercles:
      real*8    fracuni(2000)
*  Coor. des points d'intersection cercle-pixel:
      real*4    arc(2,4)
*  Azimuts des points d'intersection:
      real*4    azim1a(1000), azim1b(1000)
      real*4    azim2a(1000), azim2b(1000)
      real*4    azunia(2000), azunib(2000)
	include 'pppcore.inc'
	include 'picdir.inc'
	equivalence (iscr(1),brute(1,1)),(iscr(250001),poidsval(1,1)),
     *	            (iscr(500001),vallis(1,1)),(iscr(750001),ii1(1)),
     *	            (iscr(751001),jj1(1)),(iscr(752001),ii2(1)),
     *	            (iscr(753001),jj2(1)),(iscr(754001),iiuni(1)),
     *	            (iscr(756001),jjuni(1)),(iscr(758001),frac1(1)),
     *	            (iscr(760001),frac2(1)),(iscr(762001),fracuni(1)),
     *	            (iscr(770001),azim1a(1)),(iscr(772001),azim2a(1)),
     *	            (iscr(774001),azim1b(1)),(iscr(776001),azim2b(1)),
     *	            (iscr(778001),azunia(1)),(iscr(784001),azunib(1))
	common/iodev/idin,idout,idhis
*  Valeur retournee par arc_in_pix indiquant le n. de coins
*  interieurs au cercle:
      integer*2 ncoins

      real*8 tmp8, reste
      real*8 fracsom, fractot, fracfrac
      real*8 valmoy, valsom, valtot

      i = 1
      j = 2

c
c  axis values
	naxe1=nii(nfr)
	naxe2=njj(nfr)      
	if (naxe1.gt.250.or.naxe2.gt.250) then
	  write(idout,*) ' +++ size too large (max size 250X250) +++'
	  return
	end if
 2414 write(idout,2) ci,cj
 2	format(' Enter centroid co-ord. [dflt:',2f7.2,']: ',$)
	call read2r(ci,cj,*999,*2414)
      write(idout,3)
 3	format(' Enter sets of Radius (>1.0) & angle of smoothing sectors,'/
     *         '    ( enter 0 0 to end):')

      do 2418 k=1,20
 2415   write(idout,'('' ('',i2,''): '',$)') k
	call read2r(rayon(k),azim(k),*999,*2415)
        if (       k .gt. 2
     +      .and.  rayon(k) .eq. 0.
     +      .and.  azim(k)  .eq. 0. ) goto 2419
        if ( k .eq. 1 ) then
          if (rayon(1) .lt. 1.0) goto 2415
c         azimin = 2.*ASIND(0.708/rayon(1))
          azimin = 2.*ASIN(0.708/rayon(1))
          if (azim(1) .lt. azimin) azim(1)=azimin
        else if (     rayon(k) .le. rayon(k-1)
     +           .or. azim(k)  .lt. azim(k-1)  ) then
          goto 2415
        endif
        nray = k
 2418 continue
 2419 continue

 2424 write(idout,4) pas
 4	format('  Enter size of smoothing ring,  <1.0, >0.01 pix'
     *        ,' [dflt:',f6.3,']: ',$)
	call read1r(pas,*999,*2424)
      if ( pas .gt. 1.0  .or.  pas .lt. 0.001 ) goto 2424

      do 3218 jj=1,naxe2
        do 3214 ii=1,naxe1
          brute(ii,jj) = image(ii,jj)
          vallis(ii,jj) = 0.d0
          poidsval(ii,jj) = 0.d0
 3214   continue
 3218 continue

**********************************************************************
*   Pour chaque intervalle de rayon dans lequel on fait un lissage,
*   calcul des valeurs associees aux segments de pixels interceptes
*   aux differents azimuts.

      ncercles= NINT( (rayon(nray)-rayon(1))/pas )

      do 6888 icercle= 1, ncercles

        ray2 = rayon(1) + icercle*pas
        ray1 = ray2 - pas
        raymoy = (ray1 + ray2)/2.0

*           Demie-etendue azim. correspondant a "raymoy"

        do 6128 k=2,nray
          if ( raymoy .le. rayon(k) ) then
            demazim=(  azim(k-1)
     +               +  (azim(k)-azim(k-1))
     +                 *(raymoy-rayon(k-1))
     +                 /(rayon(k)-rayon(k-1)) ) / 2.0
            goto 6129
          endif
 6128   continue
        demazim=azim(nray)/2.0
 6129   continue

*  On construit des listes ordonnees en azimut commencant a -180 deg.
*  pour les pixels interceptes en parcourant chacun des cercles.

*  Ces listes contiennent les coordonnees ii,jj du pixel intercepte,
*  la fraction du pixel a l'interieur des cercles "ray1" et "ray2",
*  et les azimuts limites des arcs des cercles "ray1" et "ray2"
*  contenus a l'interieur du pixel.

*  Pour trouver les pixels interceptes par un cercle, on commence
*  par trouver le pixel (NINT(ci-ray),NINT(cj)) qui correspond 
*  a un azimut de -180 deg.. Ensuite on obtient les coordonnees du 
*  point d'intersection du cercle avec le cote du pixel, dans le sens
*  d'un azimut croissant (antihoraire).  A l'aide de ces coordonnees
*  on trouve le (ii,jj) du pixel suivant intercepte par le cercle.

*  Une complication provient de ce que les cercles "ray1" et "ray2"
*  n'interceptent pas toujours les memes pixels.  On cree donc une
*  liste combinee a partir des listes des pixels interceptes par
*  chacun des cercles.

        if ( icercle .eq. 1 ) then

*   On part du pixel a -180.  Si le cercle intercepte 2 pixels
*   dans cette direction, on doit commencer avec le pixel interieur.

            ii1(1) = NINT(ci-ray1)
            jj1(1) = NINT(cj)
            xi = ii1(1)
            xj = jj1(1)
            call arc_in_pix(ci,cj,xi,xj,ray1,frac,arc,ncoins)
            if ( ncoins .eq. 0 ) then
              ii1(1) = ii1(1) + 1
              jj1(1) = jj1(1)
              xi = ii1(1)
              xj = jj1(1)
              call arc_in_pix(ci,cj,xi,xj,ray1,frac,arc,ncoins)
            endif

            frac1(1) = frac
        
            tmpx = arc(i,1) - ci
            tmpy = arc(j,1) - cj
            tmp = SQRT(tmpx**2 + tmpy**2)
c           azim1a(1) = sign(acosd(tmpx/tmp), tmpy)
            azim1a(1) = sign(acos(tmpx/tmp), tmpy)
            if ( azim1a(1) .ge. 0. ) azim1a(1) = azim1a(1) - 360.
              itmp = 2
              if (     abs(arc(i,3)) .gt. 0.
     +            .or. abs(arc(j,3)) .gt. 0.
     +            .or. abs(arc(i,4)) .gt. 0.
     +            .or. abs(arc(j,4)) .gt. 0. ) itmp = 4
            tmpx = arc(i,itmp) - ci
            tmpy = arc(j,itmp) - cj
            tmp = SQRT(tmpx**2 + tmpy**2)
c           azim1b(1) = sign(acosd(tmpx/tmp), tmpy)
            azim1b(1) = sign(acos(tmpx/tmp), tmpy)
            if ( azim1b(1) .ge. 0. ) azim1b(1) = azim1b(1) - 360.

          do 6248 k=2, 2*(NAXEI+NAXEJ)

*               Coor. du pixel suivant le long du cercle
*              Cercle passe par inters. de 4 pixels? arc(,2) anti-hor.
*                donc seulement 2 des 3 pix. adjacents possibles
            if ( abs(abs(arc(i,2)-xi) - abs(arc(j,2)-xj)) .lt. 1.e-5 )
     +        then
              if      ( arc(i,2) .lt. xi .and. arc(j,2) .lt. xj ) then 
                jj1(k) = jj1(k-1) - 1
                ii1(k) = ii1(k-1)
                if ( (arc(j,2) - cj) .gt. 2.e-5 ) ii1(k) = ii1(k)-1
              else if ( arc(i,2) .gt. xi .and. arc(j,2) .lt. xj ) then
                ii1(k) = ii1(k-1) + 1
                jj1(k) = jj1(k-1)
                if ( (ci - arc(i,2)) .gt. 2.e-5 ) jj1(k) = jj1(k)-1
              else if ( arc(i,2) .gt. xi .and. arc(j,2) .gt. xj ) then
                jj1(k) = jj1(k-1) + 1
                ii1(k) = ii1(k-1)
                if ( (cj - arc(j,2)) .gt. 2.e-5 ) ii1(k) = ii1(k)+1
              else if ( arc(i,2) .lt. xi .and. arc(j,2) .gt. xj ) then
                ii1(k) = ii1(k-1) - 1
                jj1(k) = jj1(k-1)
                if ( (arc(i,2) - ci) .gt. 2.e-5 ) jj1(k) = jj1(k)+1
              endif
              
            else if ( abs(arc(i,2)-xi) .gt. abs(arc(j,2)-xj) ) then
              ii1(k) = NINT(2.0*arc(i,2) - xi)
              jj1(k) = jj1(k-1)
            else
              ii1(k) = ii1(k-1)
              jj1(k) = NINT(2.0*arc(j,2) - xj)
            endif

*                Le cercle est complet?
            if ( k .gt. 4 ) then
              if (     (ii1(k) .eq. ii1(1) .and. jj1(k) .eq. jj1(1))
     +            .or. (ii1(k) .eq. ii1(2) .and. jj1(k) .eq. jj1(2)) )
     +        then
                 nray1 = k-1
                 goto 6249
              endif
            endif

            xi = ii1(k)
            xj = jj1(k)
            call arc_in_pix(ci,cj,xi,xj,ray1,frac,arc,ncoins)

*             Si revenu au pix. precedant sauter au suivant

            if (      k .gt. 2
     +          .and. ii1(k) .eq. ii1(k-2)
     +          .and. jj1(k) .eq. jj1(k-2) ) then

*             Attention a l'incertitude d'arrond. en cas de tangence
              itmp = 2
              if (     abs(arc(i,3)) .gt. 0.
     +            .or. abs(arc(j,3)) .gt. 0.
     +            .or. abs(arc(i,4)) .gt. 0.
     +            .or. abs(arc(j,4)) .gt. 0. ) itmp = 4
              if ( abs(arc(i,itmp)-xi) .gt. abs(arc(j,itmp)-xj) ) then
                ii1(k) = NINT(2.0*arc(i,itmp) - xi)
*               jj1(k) = jj1(k)
              else
*               ii1(k) = ii1(k)
                jj1(k) = NINT(2.0*arc(j,itmp) - xj)
              endif
              
              if ( k .gt. 4 ) then
                if (     (ii1(k) .eq. ii1(1) .and. jj1(k) .eq. jj1(1))
     +              .or. (ii1(k) .eq. ii1(2) .and. jj1(k) .eq. jj1(2)) )
     +          then
                   nray1 = k-1
                   goto 6249
                endif
              endif

              xi = ii1(k)
              xj = jj1(k)
              call arc_in_pix(ci,cj,xi,xj,ray1,frac,arc,ncoins)
            endif

            frac1(k) = frac

            tmpx = arc(i,1) - ci
            tmpy = arc(j,1) - cj
            tmp = SQRT(tmpx**2 + tmpy**2)
c           azim1a(k) = sign(acosd(tmpx/tmp), tmpy)
            azim1a(k) = sign(acos(tmpx/tmp), tmpy)
            if (       k .eq. 2
     +          .and.  azim1a(2) .gt. 0. ) azim1a(2) = azim1a(2) - 360.
              itmp = 2
              if (     abs(arc(i,3)) .gt. 0.
     +            .or. abs(arc(j,3)) .gt. 0.
     +            .or. abs(arc(i,4)) .gt. 0.
     +            .or. abs(arc(j,4)) .gt. 0. ) itmp = 4
            tmpx = arc(i,itmp) - ci
            tmpy = arc(j,itmp) - cj
            tmp = SQRT(tmpx**2 + tmpy**2)
c           azim1b(k) = sign(acosd(tmpx/tmp), tmpy)
            azim1b(k) = sign(acos(tmpx/tmp), tmpy)
            if (       k .eq. 2
     +          .and.  azim1b(2) .gt. 0. ) azim1b(2) = azim1b(2) - 360.

 6248     continue
 6249     continue

******************************************
        else if ( icercle .gt. 1 ) then

          do 6268 k = 1, nray2
            ii1(k) = ii2(k)
            jj1(k) = jj2(k)
            frac1(k) = frac2(k)
            azim1a(k) = azim2a(k)
            azim1b(k) = azim2b(k)
 6268     continue
          nray1 = nray2

        endif

*   On part du pixel a -180.  Si le cercle intercepte 2 pixels
*   dans cette direction, on doit commencer avec le pixel interieur.

          ii2(1) = NINT(ci-ray2)
          jj2(1) = NINT(cj)
          xi = ii2(1)
          xj = jj2(1)
          call arc_in_pix(ci,cj,xi,xj,ray2,frac,arc,ncoins)
          if ( ncoins .eq. 0 ) then
            ii2(1) = ii2(1) + 1
            jj2(1) = jj2(1)
            xi = ii2(1)
            xj = jj2(1)
            call arc_in_pix(ci,cj,xi,xj,ray2,frac,arc,ncoins)
          endif

          frac2(1) = frac
        
          tmpx = arc(i,1) - ci
          tmpy = arc(j,1) - cj
          tmp = SQRT(tmpx**2 + tmpy**2)
c         azim2a(1) = sign(acosd(tmpx/tmp), tmpy)
          azim2a(1) = sign(acos(tmpx/tmp), tmpy)
          if ( azim2a(1) .ge. 0. ) azim2a(1) = azim2a(1) - 360.
            itmp = 2
            if (     abs(arc(i,3)) .gt. 0.
     +          .or. abs(arc(j,3)) .gt. 0.
     +          .or. abs(arc(i,4)) .gt. 0.
     +          .or. abs(arc(j,4)) .gt. 0. ) itmp = 4
          tmpx = arc(i,itmp) - ci
          tmpy = arc(j,itmp) - cj
          tmp = SQRT(tmpx**2 + tmpy**2)
c         azim2b(1) = sign(acosd(tmpx/tmp), tmpy)
          azim2b(1) = sign(acos(tmpx/tmp), tmpy)
          if ( azim2b(1) .ge. 0. ) azim2b(1) = azim2b(1) - 360.

        do 6348 k=2, 2*(NAXEI+NAXEJ)

*              Coor. du pixel suivant le long du cercle
*              Cercle passe par inters. de 4 pixels? arc(,2) anti-hor.
*                donc seulement 2 des 3 pix. adjacents possibles

          if ( abs(abs(arc(i,2)-xi) - abs(arc(j,2)-xj)) .lt. 1.e-5 )
     +      then
            if      ( arc(i,2) .lt. xi .and. arc(j,2) .lt. xj ) then 
              jj2(k) = jj2(k-1) - 1
              ii2(k) = ii2(k-1)
              if ( (arc(j,2) - cj) .gt. 2.e-5 ) ii2(k) = ii2(k)-1
            else if ( arc(i,2) .gt. xi .and. arc(j,2) .lt. xj ) then
              ii2(k) = ii2(k-1) + 1
              jj2(k) = jj2(k-1)
              if ( (ci - arc(i,2)) .gt. 2.e-5 ) jj2(k) = jj2(k)-1
            else if ( arc(i,2) .gt. xi .and. arc(j,2) .gt. xj ) then
              jj2(k) = jj2(k-1) + 1
              ii2(k) = ii2(k-1)
              if ( (cj - arc(j,2)) .gt. 2.e-5 ) ii2(k) = ii2(k)+1
            else if ( arc(i,2) .lt. xi .and. arc(j,2) .gt. xj ) then
              ii2(k) = ii2(k-1) - 1
              jj2(k) = jj2(k-1)
              if ( (arc(i,2) - ci) .gt. 2.e-5 ) jj2(k) = jj2(k)+1
            endif
              
          else if ( abs(arc(i,2)-xi) .gt. abs(arc(j,2)-xj) ) then
            ii2(k) = NINT(2.0*arc(i,2) - xi)
            jj2(k) = jj2(k-1)
          else
            ii2(k) = ii2(k-1)
            jj2(k) = NINT(2.0*arc(j,2) - xj)
          endif

*                Le cercle est complet?
          if ( k .gt. 4 ) then
            if (     (ii2(k) .eq. ii2(1) .and. jj2(k) .eq. jj2(1))
     +          .or. (ii2(k) .eq. ii2(2) .and. jj2(k) .eq. jj2(2)) )
     +      then
               nray2 = k-1
               goto 6349
            endif
          endif

          xi = ii2(k)
          xj = jj2(k)
          call arc_in_pix(ci,cj,xi,xj,ray2,frac,arc,ncoins)

*             Si revenu au pixel precedant sauter au suivant

          if (      k .gt. 2
     +        .and. ii2(k) .eq. ii2(k-2)
     +        .and. jj2(k) .eq. jj2(k-2) ) then

*           Attention a l'incertitude d'arrond. en cas de tangence
            itmp = 2
            if (     abs(arc(i,3)) .gt. 0.
     +          .or. abs(arc(j,3)) .gt. 0.
     +          .or. abs(arc(i,4)) .gt. 0.
     +          .or. abs(arc(j,4)) .gt. 0. ) itmp = 4
            if ( abs(arc(i,itmp)-xi) .gt. abs(arc(j,itmp)-xj) ) then
              ii2(k) = NINT(2.0*arc(i,itmp) - xi)
*             jj2(k) = jj2(k)
            else
*             ii2(k) = ii2(k)
              jj2(k) = NINT(2.0*arc(j,itmp) - xj)
            endif
           
            if ( k .gt. 4 ) then   
              if (     (ii2(k) .eq. ii2(1) .and. jj2(k) .eq. jj2(1))
     +            .or. (ii2(k) .eq. ii2(2) .and. jj2(k) .eq. jj2(2)) )
     +        then
                nray2 = k-1
                goto 6349
              endif
            endif

            xi = ii2(k)
            xj = jj2(k)
            call arc_in_pix(ci,cj,xi,xj,ray2,frac,arc,ncoins)
          endif

          frac2(k) = frac

          tmpx = arc(i,1) - ci
          tmpy = arc(j,1) - cj
          tmp = SQRT(tmpx**2 + tmpy**2)
c         azim2a(k) = sign(acosd(tmpx/tmp), tmpy)
          azim2a(k) = sign(acos(tmpx/tmp), tmpy)
          if (       k .eq. 2
     +        .and.  azim2a(2) .gt. 0. ) azim2a(2) = azim2a(2) - 360.
            itmp = 2
            if (     abs(arc(i,3)) .gt. 0.
     +          .or. abs(arc(j,3)) .gt. 0.
     +          .or. abs(arc(i,4)) .gt. 0.
     +          .or. abs(arc(j,4)) .gt. 0. ) itmp = 4
          tmpx = arc(i,itmp) - ci
          tmpy = arc(j,itmp) - cj
          tmp = SQRT(tmpx**2 + tmpy**2)
c         azim2b(k) = sign(acosd(tmpx/tmp), tmpy)
          azim2b(k) = sign(acos(tmpx/tmp), tmpy)
          if (       k .eq. 2
     +        .and.  azim2b(2) .gt. 0. ) azim2b(2) = azim2b(2) - 360.

 6348   continue
 6349   continue

**********************************************************************

        kray1 = 1
        kray2 = 1
        kuni = 1

 6430   if (      kray1 .le. nray1
     +      .and. kray2 .le. nray2 ) then

*  Base sur le fait qu'un pixel intercepte par les 2 rayons ne
*  peut avoir un point d'intersection avec un rayon, d'azimut 
*  plus petit que celui du point d'intersection d'un pixel precedant
*  le pixel commun dans la liste de l'autre rayon.
*  Donc un pixel commun sera toujours reconnu comme tel.
          if (      ii1(kray1) .eq. ii2(kray2)
     +        .and. jj1(kray1) .eq. jj2(kray2) ) then
            iiuni(kuni) = ii1(kray1)
            jjuni(kuni) = jj1(kray1)
            azunia(kuni) = (azim1a(kray1) + azim2a(kray2)) / 2.0
            azunib(kuni) = (azim1b(kray1) + azim2b(kray2)) / 2.0
            fracuni(kuni) = frac2(kray2) - frac1(kray1)
            kray1 = kray1 + 1
            kray2 = kray2 + 1
            if ( fracuni(kuni) .gt. 1.d-6 ) kuni = kuni + 1
          else if ( azim1a(kray1) .le. azim2a(kray2) ) then 
            iiuni(kuni) = ii1(kray1)
            jjuni(kuni) = jj1(kray1)
            azunia(kuni) = azim1a(kray1)
            azunib(kuni) = azim1b(kray1)
            fracuni(kuni) = 1.0 - frac1(kray1)
            kray1 = kray1 + 1
            if ( fracuni(kuni) .gt. 1.d-6 ) kuni = kuni + 1
          else if ( azim2a(kray2) .lt. azim1a(kray1) ) then
            iiuni(kuni) = ii2(kray2)
            jjuni(kuni) = jj2(kray2)
            azunia(kuni) = azim2a(kray2)
            azunib(kuni) = azim2b(kray2)
            fracuni(kuni) = frac2(kray2)
            kray2 = kray2 + 1
            if ( fracuni(kuni) .gt. 1.d-6 ) kuni = kuni + 1
          endif

        goto 6430

        endif

        if ( kray1 .le. nray1 ) then
          do 6448 k=kray1, nray1
            fracuni(kuni) = 1.0 - frac1(k)
            if ( fracuni(kuni) .gt. 1.d-6 ) then
              iiuni(kuni) = ii1(k)
              jjuni(kuni) = jj1(k)
              azunia(kuni) = azim1a(k)
              azunib(kuni) = azim1b(k)
              kuni = kuni + 1
            endif
 6448     continue 
        endif

        if ( kray2 .le. nray2 ) then
          do 6458 k=kray2, nray2
            fracuni(kuni) = frac2(k)
            if ( fracuni(kuni) .gt. 1.d-6 ) then
              iiuni(kuni) = ii2(k)
              jjuni(kuni) = jj2(k)
              azunia(kuni) = azim2a(k)
              azunib(kuni) = azim2b(k)
              kuni = kuni + 1
            endif
 6458     continue 
        endif

        nuni = kuni - 1

*      if ( icercle .eq. icertst ) then
*      write(nur,'(1x)')
*      do k=1,nuni
*        write(nur,'(1x,2i6,e10.2,2f11.4)') iiuni(k),jjuni(k),fracuni(k)
*     +                                    ,azunia(k),azunib(k)
*      enddo
*      endif

*      Allongement de la liste par repetition des pixels pour 
*      faciliter la moyenne.

        do 6468 k= 1, nuni
          itmp = nuni+k
          iiuni(itmp) = iiuni(k)
          jjuni(itmp) = jjuni(k)
          fracuni(itmp) = fracuni(k)
          azunia(itmp) = azunia(k) + 360.
          azunib(itmp) = azunib(k) + 360.
 6468   continue

*****************************************************************
*       Trouver l'indice du pixel a 0 deg. dans la liste combinee
*       et calculer les valeurs moy. dans les arcs definis par
*       "ray1", "ray2", "azmin", et "azmax" a partir de ce pixel.

        do 6518 k= 1, nuni
          if ( azunia(k) .gt. 0. ) then
            k1 = k - 1
            goto 6519
          endif
 6518   continue
 6519   continue

          azkk = 0.5*(azunia(k1)+azunib(k1))
          azmin = azkk - demazim
          azmax = azkk + demazim
          kmin = k1
          kmax = k1
          valsom = fracuni(k1)*brute(iiuni(k1),jjuni(k1))
          fracsom = fracuni(k1)

*    Somme sur les segments de pixels qui sont entierement
*    compris entre azmin et azmax.

          do 6524 k= k1-1, 1, -1
            if ( azunia(k) .lt. azmin ) then
              kmin = k+1
              goto 6525
            endif
            valsom =  valsom
     +              + fracuni(k)*brute(iiuni(k),jjuni(k))
            fracsom = fracsom + fracuni(k)
 6524     continue
 6525     continue
          do 6528 k= k1+1, 2*nuni
            if ( azunib(k) .gt. azmax ) then
              kmax = k-1
              goto 6529
            endif
            valsom =  valsom
     +              + fracuni(k)*brute(iiuni(k),jjuni(k))
            fracsom = fracsom + fracuni(k)
 6528     continue
 6529     continue

          valtot = valsom
          fractot = fracsom

*       Ajouter la contribution des pixels partiellement dans l'intervalle

          do 6534 k= kmin-1, kmin-3, -1
            if (      azunib(k) .gt. azmin
     +          .and. azunib(k) .gt. azunia(k) ) then
              fracfrac =   fracuni(k)
     +                   * (azunib(k)-azmin)/(azunib(k)-azunia(k))
              if ( fracfrac .gt. 1.d-8 )  then
                valtot =  valtot
     +                  + fracfrac*brute(iiuni(k),jjuni(k))
                fractot = fractot + fracfrac
              endif     
            endif
 6534     continue
          do 6538 k= kmax+1, kmax+3
            if (      azunia(k) .lt. azmax
     +          .and. azunib(k) .gt. azunia(k) ) then
              fracfrac =   fracuni(k)
     +                   * (azmax-azunia(k))/(azunib(k)-azunia(k))
              if ( azunib(k) .lt. azmax ) fracfrac = fracuni(k)
              if ( fracfrac .gt. 1.d-8 )  then
                valtot =  valtot
     +                  + fracfrac*brute(iiuni(k),jjuni(k))
                fractot = fractot + fracfrac
              endif
            endif
 6538     continue

          if ( fractot .gt. 1.d-8 )  then
            valmoy = valtot/fractot
            iikk = iiuni(k1)
            jjkk = jjuni(k1)
            tmp8 = fracuni(k1)
            poidsval(iikk,jjkk) = poidsval(iikk,jjkk) + tmp8
            vallis(iikk,jjkk) = vallis(iikk,jjkk) + valmoy*tmp8
          endif
 
        do 6588 kk = k1+1, k1+nuni-1

          azkk = 0.5*(azunia(kk)+azunib(kk))
          azmin = azkk - demazim
          azmax = azkk + demazim

*          Soustraire de "valsom" et "fracsom" les segments qui ne
*          sont plus entierement compris dans l'intervalle

 6564     if ( azunia(kmin) .lt. azmin ) then
            valsom =  valsom
     +              - fracuni(kmin)*brute(iiuni(kmin),jjuni(kmin))
            fracsom = fracsom - fracuni(kmin)
            kmin = kmin + 1            
            goto 6564
          endif

*          Ajouter a "valsom" et "fracsom" les segments qui sont
*          maintenant entierement compris dans l'intervalle

 6568     if ( azunib(kmax+1) .lt. azmax ) then
            kmax = kmax + 1
*            if ( kmax .lt. 1  .or.  kmax .gt. 2*nuni ) then
*              write(nur,6567) icercle, kk, nuni, kmax
* 6567         format(' icer',i3,' kk',i3,' nuni',i3,' kmax',i5)
*            else
*             if( iiuni(kmax) .lt. 1 .or. iiuni(kmax) .gt. naxe1
*     +          .or. jjuni(kmax) .lt. 1 .or. jjuni(kmax) .gt. naxe2)
*     +       then              
*              write(nur,6569) icercle, kk, nuni, kmax, iiuni(kmax)
*     +                        , jjuni(kmax)
* 6569       format(' icer',i3,' kk',i3,' nuni',i3,' kmx',i5
*     +             ,' ii,jj',2i3)
*             else
            valsom =  valsom
     +              + fracuni(kmax)*brute(iiuni(kmax),jjuni(kmax))
            fracsom = fracsom + fracuni(kmax)
*             endif
*            endif
            goto 6568
          endif

          valtot = valsom
          fractot = fracsom

*       Ajouter la contribution des pixels partiellement dans l'intervalle

          do 6574 k= kmin-1, kmin-3, -1
            if (      azunib(k) .gt. azmin
     +          .and. azunib(k) .gt. azunia(k) ) then
              fracfrac =   fracuni(k)
     +                   * (azunib(k)-azmin)/(azunib(k)-azunia(k))
              if ( fracfrac .gt. 1.d-8 ) then
                valtot =  valtot
     +                  + fracfrac*brute(iiuni(k),jjuni(k))
                fractot = fractot + fracfrac
              endif
            endif
 6574     continue
          do 6578 k= kmax+1, kmax+3
            if (      azunia(k) .lt. azmax
     +          .and. azunib(k) .gt. azunia(k) ) then
              fracfrac =   fracuni(k)
     +                   * (azmax-azunia(k))/(azunib(k)-azunia(k))
              if ( azunib(k) .lt. azmax ) fracfrac = fracuni(k)
              if ( fracfrac .gt. 1.d-8 ) then
                valtot =  valtot
     +                  + fracfrac*brute(iiuni(k),jjuni(k))
                fractot = fractot + fracfrac
              endif
            endif
 6578     continue
 
          if ( fractot .gt. 1.d-8 ) then
            valmoy = valtot/fractot
            iikk = iiuni(kk)
            jjkk = jjuni(kk)
            tmp8 = fracuni(kk)

*            if ( iikk .eq. iitst .and. jjkk .eq. jjtst ) then
*              write(nur,6579) icercle,kk, valtot, fractot, tmp8
* 6579         format(' icer',i3,' kk',i3
*     +              ,' valtot,fractot,frac',3d12.5)
*            endif

            poidsval(iikk,jjkk) = poidsval(iikk,jjkk) + tmp8
            vallis(iikk,jjkk) = vallis(iikk,jjkk) + valmoy*tmp8
          endif

 6588   continue

 6888 continue

*******************************************************************
*   Dans la boucle ci-dessous, la difference entre la valeur calculee
*   d'un pixel et la valeur entiere qui lui est attribuee est gardee
*   et accumulee. Lorsque la valeur absolue de cette difference
*   depasse 0.5, la valeur du pixel courant est augmentee ou diminuee
*   de 1, et la valeur de la difference ajustee en consequence.
*   
      imagsom = 0
      imagdif = 0
      reste = 0.d0

      do 7488 jj=1, naxe2
        do 7448 ii=1, naxe1
            imagsom = imagsom + image(ii,jj)
          if ( poidsval(ii,jj) .gt. 1.d-8 ) then
            imagdif = imagdif + image(ii,jj)
            tmp8 = vallis(ii,jj) + brute(ii,jj)*(1.d0-poidsval(ii,jj))
     +            + reste
c           image(ii,jj) = IIDNNT(tmp8)
            image(ii,jj) = nint(tmp8)
            reste = tmp8 - image(ii,jj)
            imagdif = imagdif - image(ii,jj)
          endif
 7448    continue
 7488 continue

      write(idout,'('' sum before - sum after:'',i8)') imagdif
      tmp4 = imagdif
      tmp4 = tmp4 / imagsom
      write(idout,'('' Frac. diff of total flux: '',e9.2)') tmp4
 999	return
      end   

****************************************************************
*                                                              *
      subroutine arc_in_pix(ci,cj,xi,xj,rad,fracint,arc,ierr)
*                                                              *
*  D. Nadeau                                                   *
*  Universite de Montreal                     9 JUILLET 1989   *
*                                                              *
*   Trouve la fraction "fracint" du pixel centre a xi,xj       *
*   qui se trouve a l'interieur du cercle de rayon "rad"       *
*   centre a ci,cj.                                            *
*                                                              *
*   Donne aussi, dans "arc", les coordonnees de points de      *
*   l'arc qui intersectent les bords du pixel, le 2e point     *
*   etant dans la direction anti-horaire par rapport au 1er,   *
*   selon la convention habituelle:                            *
*                x>0, y=0  ->  theta=0                         *
*                x=0, y>0  ->  theta=90 deg.                   *
*   Dans le cas ou il y a 4 points d'intersection avec les     *
*   bords, on met les 4 points dans "arc"; dans les autres     *
*   cas, "arc(,3)" et "arc(,4)" sont mis a zero.               *
*   Dans le cas ou il n'y a pas d'intersection du cercle avec  *
*   le pixel et ou le point le plus pres du cercle n'est pas   *
*   un des coins, on met dans "arc(,1)" et "arc(,2)" les       *
*   coordonnees du point le plus pres.                         *
*                                                              *
*   Les cotes du pixel ont une longueur egale a 1.0            *
*   et "rad" doit etre plus grand que 1.0.                     *
*                                                              *
*              ierr = -1     :     "rad" < 1.0                 *
*           ierr = 0,1,2,3,4 : n. de coins inter. au cercle    *
*                                                              *
****************************************************************
      integer*2 ierr
      integer*2 itmp, jtmp

      real*4 ci, cj
      real*4 xi, xj
      real*4 xa(2,2),x(2,2)
      real*4 x2(2,2)
      real*4 arc(2,4)
      real*4 tmp,tmp4
      real*4 rad, rad2

      real*8 tmp8

      i = 1
      j = 2

      arc(i,3) = 0.
      arc(i,4) = 0.
      arc(j,3) = 0.
      arc(j,4) = 0.
      rad = abs(rad)
      rad2 = rad**2

      if ( rad .lt. 1.0 ) then
        ierr = -1
        fracint = 9999.
        return
      endif

*  Calcul des distances des 4 coins du pixel 
*  par rapport a ci,cj.
*  1er indice designe i ou j, 2e indice designe i1 ou i2, etc.

      xa(i,1)  = (xi - ci) - 0.5
      x2(i,1) = xa(i,1)**2
      xa(i,2)  = (xi - ci) + 0.5
      x2(i,2) = xa(i,2)**2
      xa(j,1)  = (xj - cj) - 0.5
      x2(j,1) = xa(j,1)**2
      xa(j,2)  = (xj - cj) + 0.5
      x2(j,2) = xa(j,2)**2

      if ( x2(i,1) .le. x2(i,2) )  then
        mini = 1
        maxi = 2
      else
        mini = 2
        maxi = 1
      endif

      if ( x2(j,1) .le. x2(j,2) )  then 
        minj = 1
        maxj = 2
      else
        minj = 2
        maxj = 1
      endif

*  Pour que les termes sommes dans "fracint" aient les bons signes
*  on fait un changement de coordonnees tel que le coin le plus 
*  eloigne du centre ait des coordonnees positives. On garde les
*  coordonnees originales pour le calcul des extremites de l'arc.

      if ( xa(i,maxi) .lt. 0. ) then
        x(i,mini) = -xa(i,mini)
        x(i,maxi) = -xa(i,maxi)
      else
        x(i,mini) =  xa(i,mini)
        x(i,maxi) =  xa(i,maxi)
      endif

      if ( xa(j,maxj) .lt. 0. ) then
        x(j,minj) = -xa(j,minj)
        x(j,maxj) = -xa(j,maxj)
      else
        x(j,minj) =  xa(j,minj)
        x(j,maxj) =  xa(j,maxj)
      endif

*  Determination des coordonnees des distances intermediaires
*  pres du minimum (i2,j2) et du maximum (i3,j3)

      if (      x2(i,mini) + x2(j,maxj)
     +     .le. x2(i,maxi) + x2(j,minj)  ) then
        i2 = mini
        i3 = maxi
        j2 = maxj
        j3 = minj
      else
        i2 = maxi
        i3 = mini
        j2 = minj
        j3 = maxj
      endif

*  SI TOUS LES COINS SONT A L'INTERIEUR

      if ( (x2(i,maxi) + x2(j,maxj)) .le. rad2 ) then
          fracint = 1.0
          ierr = 4
          arc(i,1) = 0.
          arc(i,2) = 0.
          arc(j,1) = 0.
          arc(j,2) = 0.
          return
      endif

*  SI TOUS LES COINS SONT A L'EXTERIEUR
*  La seule possibilite que le cercle penetre a l'interieur
*  du pixel est que l'un des axes i ou j, passant par (ci,cj)
*  traverse le cote du pixel entre les 2 coins les plus pres
*  a une distance plus petite que "rad".  Dans ce cas,
*  par symetrie, on peut integrer de 0 a xlim2 et multiplier
*  le resultat par 2.

      if ( (x2(i,mini) + x2(j,minj)) .ge. rad2 ) then

        ierr = 0

        if     (            j2 .eq. minj
     +          .and.     rad2 .ge. x2(j,minj)   
     +          .and. x(i,mini)/x(i,maxi) .lt. 0. ) then

          xlim1 = 0.
          xlim2 = SQRT(rad2 - x2(j,minj))
          ylim2 = x(j,minj)

          arc(j,1) = xa(j,minj) + cj
          arc(j,2) = xa(j,minj) + cj
          if ( xa(j,minj) .ge. 0. ) then
            arc(i,1) =  xlim2 + ci
            arc(i,2) = -xlim2 + ci
          else
            arc(i,1) = -xlim2 + ci
            arc(i,2) =  xlim2 + ci
          endif

          theta2 = asin( xlim2/rad )
          fracint =   rad2 * theta2
     +              - xlim2*ylim2

        else if (             i2 .eq. mini
     +           .and.     rad2 .ge. x2(i,mini)   
     +           .and. x(j,minj)/x(j,maxj) .lt. 0. ) then

          xlim1 = 0.
          xlim2 = SQRT(rad2 - x2(i,mini))
          ylim2 = x(i,mini)

          arc(i,1) = xa(i,mini) + ci
          arc(i,2) = xa(i,mini) + ci
          if ( xa(i,mini) .ge. 0. ) then
            arc(j,1) = -xlim2 + cj
            arc(j,2) =  xlim2 + cj
          else
            arc(j,1) =  xlim2 + cj
            arc(j,2) = -xlim2 + cj
          endif

          theta2 = asin( xlim2/rad )
          fracint =   rad2 * theta2
     +              - xlim2*ylim2

        else

          fracint = 0.

          if      (            j2 .eq. minj
     +             .and. x(i,mini)/x(i,maxi) .lt. 0. ) then

            arc(j,1) = xa(j,minj) + cj
            arc(j,2) = xa(j,minj) + cj
            arc(i,1) = ci
            arc(i,2) = ci

          else if (            i2 .eq. mini
     +             .and. x(j,minj)/x(j,maxj) .lt. 0. ) then

            arc(i,1) = xa(i,mini) + ci
            arc(i,2) = xa(i,mini) + ci
            arc(j,1) = cj
            arc(j,2) = cj

          endif
        endif

        return

      endif

*  SI 1 SEUL COIN EST A L'INTERIEUR

      if ( (x2(i,i2) + x2(j,j2)) .ge. rad2 ) then

        ierr = 1

*  On fait l'integrale le long de l'axe I, entre les 
*  limites  "x(i,mini)" et "SQRT(rad2 - x2(j,minj))"
*     Noter que la fonction "asin" va de -pi a +pi

        xlim1 = x(i,mini)
        xlim2 = SQRT(rad2 - x2(j,minj))
        ylim1 = SQRT(rad2 - x2(i,mini))
        ylim2 = x(j,minj)

        if ( j2 .eq. minj ) then
          if      (      xa(j,maxj) .ge. 0. 
     +             .and. xa(i,mini) .gt. xa(i,maxi) ) then
            arc(i,1) = xa(i,mini) + ci
            arc(i,2) = -xlim2 + ci
            arc(j,1) =  ylim1 + cj
            arc(j,2) = xa(j,minj) + cj
          else if (      xa(j,maxj) .ge. 0. 
     +             .and. xa(i,mini) .lt. xa(i,maxi) ) then
            arc(i,1) =  xlim2 + ci
            arc(i,2) = xa(i,mini) + ci
            arc(j,1) = xa(j,minj) + cj
            arc(j,2) =  ylim1 + cj
          else if (      xa(j,maxj) .lt. 0. 
     +             .and. xa(i,mini) .gt. xa(i,maxi) ) then
            arc(i,1) = -xlim2 + ci
            arc(i,2) = xa(i,mini) + ci
            arc(j,1) = xa(j,minj) + cj
            arc(j,2) = -ylim1 + cj
          else if (      xa(j,maxj) .lt. 0. 
     +             .and. xa(i,mini) .lt. xa(i,maxi) ) then
            arc(i,1) = xa(i,mini) + ci
            arc(i,2) =  xlim2 + ci
            arc(j,1) = -ylim1 + cj
            arc(j,2) = xa(j,minj) + cj
          endif
        else if ( i2 .eq. mini ) then
          if      (      xa(i,maxi) .ge. 0. 
     +             .and. xa(j,minj) .gt. xa(j,maxj) ) then
            arc(i,1) = xa(i,mini) + ci
            arc(i,2) =  xlim2 + ci
            arc(j,1) = -ylim1 + cj
            arc(j,2) = xa(j,minj) + cj
          else if (      xa(i,maxi) .ge. 0. 
     +             .and. xa(j,minj) .lt. xa(j,maxj) ) then
            arc(i,1) =  xlim2 + ci
            arc(i,2) = xa(i,mini) + ci
            arc(j,1) = xa(j,minj) + cj
            arc(j,2) =  ylim1 + cj
          else if (      xa(i,maxi) .lt. 0. 
     +             .and. xa(j,minj) .gt. xa(j,maxj) ) then
            arc(i,1) = -xlim2 + ci
            arc(i,2) = xa(i,mini) + ci
            arc(j,1) = xa(j,minj) + cj
            arc(j,2) = -ylim1 + cj
          else if (      xa(i,maxi) .lt. 0. 
     +             .and. xa(j,minj) .lt. xa(j,maxj) ) then
            arc(i,1) = xa(i,mini) + ci
            arc(i,2) = -xlim2 + ci
            arc(j,1) =  ylim1 + cj
            arc(j,2) = xa(j,minj) + cj
          endif
        endif

        theta1 = asin( xlim1/rad )
        theta2 = asin( xlim2/rad )
        fracint =   0.5 * rad2 * (theta2 - theta1)
     +            + 0.5 * (xlim2*ylim2 - xlim1*ylim1)
     +            - ylim2 * (xlim2 - xlim1)


        return

      endif   

*  SI 1 SEUL COIN EST A L'EXTERIEUR

      if ( (x2(i,i3) + x2(j,j3)) .le. rad2 ) then

        ierr = 3

*  On fait l'integrale le long de l'axe I, entre les 
*  limites  "SQRT(rad2 - x2(j,maxj))" et "x(i,maxi)"

        xlim1 = SQRT(rad2 - x2(j,maxj))
        xlim2 = x(i,maxi)
        ylim1 = x(j,maxj)
        ylim2 = SQRT(rad2 - x2(i,maxi))

        if ( j2 .eq. minj ) then
          if      (      xa(j,maxj) .ge. 0. 
     +             .and. xa(i,mini) .gt. xa(i,maxi) ) then
            arc(i,1) = -xlim1 + ci
            arc(i,2) = xa(i,maxi) + ci
            arc(j,1) = xa(j,maxj) + cj
            arc(j,2) =  ylim2 + cj
          else if (      xa(j,maxj) .ge. 0. 
     +             .and. xa(i,mini) .lt. xa(i,maxi) ) then
            arc(i,1) = xa(i,maxi) + ci
            arc(i,2) =  xlim1 + ci
            arc(j,1) =  ylim2 + cj
            arc(j,2) = xa(j,maxj) + cj
          else if (      xa(j,maxj) .lt. 0. 
     +             .and. xa(i,mini) .gt. xa(i,maxi) ) then
            arc(i,1) = xa(i,maxi) + ci
            arc(i,2) = -xlim1 + ci
            arc(j,1) = -ylim2 + cj
            arc(j,2) = xa(j,maxj) + cj
          else if (      xa(j,maxj) .lt. 0. 
     +             .and. xa(i,mini) .lt. xa(i,maxi) ) then
            arc(i,1) =  xlim1 + ci
            arc(i,2) = xa(i,maxi) + ci
            arc(j,1) = xa(j,maxj) + cj
            arc(j,2) = -ylim2 + cj
          endif
        else if ( i2 .eq. mini ) then
          if      (      xa(i,maxi) .ge. 0. 
     +             .and. xa(j,minj) .gt. xa(j,maxj) ) then
            arc(i,1) =  xlim1 + ci
            arc(i,2) = xa(i,maxi) + ci
            arc(j,1) = xa(j,maxj) + cj
            arc(j,2) = -ylim2 + cj
          else if (      xa(i,maxi) .ge. 0. 
     +             .and. xa(j,minj) .lt. xa(j,maxj) ) then
            arc(i,1) = xa(i,maxi) + ci
            arc(i,2) =  xlim1 + ci
            arc(j,1) =  ylim2 + cj
            arc(j,2) = xa(j,maxj) + cj
          else if (      xa(i,maxi) .lt. 0. 
     +             .and. xa(j,minj) .gt. xa(j,maxj) ) then
            arc(i,1) = xa(i,maxi) + ci
            arc(i,2) = -xlim1 + ci
            arc(j,1) = -ylim2 + cj
            arc(j,2) = xa(j,maxj) + cj
          else if (      xa(i,maxi) .lt. 0. 
     +             .and. xa(j,minj) .lt. xa(j,maxj) ) then
            arc(i,1) = -xlim1 + ci
            arc(i,2) = xa(i,maxi) + ci
            arc(j,1) = xa(j,maxj) + cj
            arc(j,2) =  ylim2 + cj
          endif
        endif

        theta1 = asin( xlim1/rad )
        theta2 = asin( xlim2/rad )
        fracint =   0.5 * rad2 * (theta2 - theta1)
     +            + 0.5 * (xlim2*ylim2 - xlim1*ylim1)
     +            - ylim1 * (xlim2 - xlim1)
     +            + 1.0

        return

      endif   

*  SI 2 COINS SONT A L'INTERIEUR ET 2 SONT A L'EXTERIEUR
*    On integre de "x(i,mini)" a "x(i,maxi)" si ce
*    sont les 2 coins a "minj" qui sont a l'interieur
*    et de "x(j,minj)" a "x(j,maxj) si ce sont
*    les coins a "mini" qui sont a l'interieur.

      ierr = 2

      if ( j2 .eq. minj ) then
        xlim1 = x(i,mini)
        xlim2 = x(i,maxi)
        ylim1 = SQRT(rad2 - x2(i,mini))
        ylim2 = SQRT(rad2 - x2(i,maxi))
        y0    = x(j,minj)

        if      (      xa(j,maxj) .ge. 0. 
     +           .and. xa(i,mini) .gt. xa(i,maxi) ) then
          arc(i,1) = xa(i,mini) + ci
          arc(i,2) = xa(i,maxi) + ci
          arc(j,1) = ylim1 + cj
          arc(j,2) = ylim2 + cj
        else if (      xa(j,maxj) .ge. 0. 
     +           .and. xa(i,mini) .lt. xa(i,maxi) ) then
          arc(i,1) = xa(i,maxi) + ci
          arc(i,2) = xa(i,mini) + ci
          arc(j,1) = ylim2 + cj
          arc(j,2) = ylim1 + cj
        else if (      xa(j,maxj) .lt. 0. 
     +           .and. xa(i,mini) .gt. xa(i,maxi) ) then
          arc(i,1) = xa(i,maxi) + ci
          arc(i,2) = xa(i,mini) + ci
          arc(j,1) = -ylim2 + cj
          arc(j,2) = -ylim1 + cj
        else if (      xa(j,maxj) .lt. 0. 
     +           .and. xa(i,mini) .lt. xa(i,maxi) ) then
          arc(i,1) = xa(i,mini) + ci
          arc(i,2) = xa(i,maxi) + ci
          arc(j,1) = -ylim1 + cj
          arc(j,2) = -ylim2 + cj
        endif
      else if ( i2 .eq. mini ) then
        xlim1 = x(j,minj)
        xlim2 = x(j,maxj)
        ylim1 = SQRT(rad2 - x2(j,minj))
        ylim2 = SQRT(rad2 - x2(j,maxj))
        y0    = x(i,mini)

        if      (      xa(i,maxi) .ge. 0. 
     +           .and. xa(j,minj) .gt. xa(j,maxj) ) then
          arc(i,1) = ylim2 + ci
          arc(i,2) = ylim1 + ci
          arc(j,1) = xa(j,maxj) + cj
          arc(j,2) = xa(j,minj) + cj
        else if (      xa(i,maxi) .ge. 0. 
     +           .and. xa(j,minj) .lt. xa(j,maxj) ) then
          arc(i,1) = ylim1 + ci
          arc(i,2) = ylim2 + ci
          arc(j,1) = xa(j,minj) + cj
          arc(j,2) = xa(j,maxj) + cj
        else if (      xa(i,maxi) .lt. 0. 
     +           .and. xa(j,minj) .gt. xa(j,maxj) ) then
          arc(i,1) = -ylim1 + ci
          arc(i,2) = -ylim2 + ci
          arc(j,1) = xa(j,minj) + cj
          arc(j,2) = xa(j,maxj) + cj
        else if (      xa(i,maxi) .lt. 0. 
     +           .and. xa(j,minj) .lt. xa(j,maxj) ) then
          arc(i,1) = -ylim2 + ci
          arc(i,2) = -ylim1 + ci
          arc(j,1) = xa(j,maxj) + cj
          arc(j,2) = xa(j,minj) + cj
        endif
      endif

      theta1 = asin( xlim1/rad )
      theta2 = asin( xlim2/rad )
      fracint =   0.5 * rad2 * (theta2 - theta1)
     +          + 0.5 * (xlim2*ylim2 - xlim1*ylim1)
     +          - y0 * (xlim2 - xlim1)

*    Si le cercle traverse le cote entre les 2 coins exter.
*    il faut soustraire la surface exterieure au pixel
*    et ajouter les points d'intersection dans "arc".
*    Le calcul est analogue au cas des 4 coins exterieurs.

        if     (            j2 .eq. minj
     +          .and.     rad2 .gt. x2(j,maxj)   
     +          .and. x(i,mini)/x(i,maxi) .lt. 0. ) then

          xlim1 = 0.
          xlim2 = SQRT(rad2 - x2(j,maxj))
          ylim2 = x(j,maxj)

          arc(i,4) = arc(i,2)
          arc(j,4) = arc(j,2)
          arc(j,2) = xa(j,maxj) + cj
          arc(j,3) = xa(j,maxj) + cj
          if ( xa(j,maxj) .ge. 0. ) then
            arc(i,2) =  xlim2 + ci
            arc(i,3) = -xlim2 + ci
          else
            arc(i,2) = -xlim2 + ci
            arc(i,3) =  xlim2 + ci
          endif

          theta2 = asin( xlim2/rad )
          fracint = fracint - rad2 * theta2
     +                      + xlim2*ylim2

        else if (             i2 .eq. mini
     +           .and.     rad2 .gt. x2(i,maxi)   
     +           .and. x(j,minj)/x(j,maxj) .lt. 0. ) then

          xlim1 = 0.
          xlim2 = SQRT(rad2 - x2(i,maxi))
          ylim2 = x(i,maxi)

          arc(i,4) = arc(i,2)
          arc(j,4) = arc(j,2)
          arc(i,2) = xa(i,maxi) + ci
          arc(i,3) = xa(i,maxi) + ci
          if ( xa(i,maxi) .ge. 0. ) then
            arc(j,2) = -xlim2 + cj
            arc(j,3) =  xlim2 + cj
          else
            arc(j,2) =  xlim2 + cj
            arc(j,3) = -xlim2 + cj
          endif

          theta2 = asin( xlim2/rad )
          fracint = fracint - rad2 * theta2
     +                      + xlim2*ylim2

        endif

 9999 return
      end   

